#include "sdk.h"
#include "KTD2061.h"

#define KTD_TWI_CH	0
#define KTD_DEVICE_ADDR		0xD0

U8 control_reg_data = 0;
U8 isel_a12 = 0;
U8 isel_a34 = 0;
U8 isel_b12 = 0;
U8 isel_b34 = 0;
U8 isel_c12 = 0;
U8 isel_c34 = 0;

bool ktd_reset()
{	
	U8 data = CON_RESET;
	char res = 0;
	
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             CONTROL_REG, data);
	
	if(res==-1)
	{
		debugprintf("reset fail. \r\n");
		return false;
	}
	
	control_reg_data = 0;
	
	isel_a12 = 0;
	isel_a34 = 0;
	isel_b12 = 0;
	isel_b34 = 0;
	isel_c12 = 0;
	isel_c34 = 0;
	return true;
}

bool ktd_enable(U8 en_mod)
{
	char  res = 0;
	
	control_reg_data &= ~(0x3 << 6);
	control_reg_data |= en_mod;		
		
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             CONTROL_REG, control_reg_data);
						 
	if(res == -1)
		return false;
	
	return true;
}

bool ktd_bright_ex(U8 b)
{
	char  res = 0;
	
	control_reg_data &= ~(0x1 << 5);
	control_reg_data |= b;
	
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             CONTROL_REG, control_reg_data);
	if(res == -1)
		return false;
	
	return true;	
}

bool ktd_cool_ex(U8 temp)
{
	char  res = 0;
	control_reg_data &= ~(0x3 << 3);
	control_reg_data |= temp;
	
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             CONTROL_REG, control_reg_data);
	if(res == -1)
		return false;
	
	return true;	
}

bool ktd_fade_rate(U8 rate)
{
	char  res = 0;
	control_reg_data &= ~(0x7 << 0);
	control_reg_data |= rate;	
	
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             CONTROL_REG, control_reg_data);
	if(res == -1)
		return false;
	
	return true;		
	
	return 0;
}

bool ktd_set_rgb0(U8 r, U8 g, U8 b)
{	
	char  res = 0;
		
	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             IRED0, r);	
	if(res == -1)
		return false;

	res = twi_write(KTD_TWI_CH, 
                             KTD_DEVICE_ADDR, 
                             IGRN0, g);	
	if(res == -1)
		return false;
		
	res = twi_write(KTD_TWI_CH,
                             KTD_DEVICE_ADDR,
                             IBLU0,b);
	if(res == -1)
		return false;
	
	return true;
}

bool ktd_set_rgb1(U8 r, U8 g, U8 b)
{	
	char res = 0;
		
	res = twi_write(KTD_TWI_CH,
                             KTD_DEVICE_ADDR,
                             IRED1,r);	
	if(res == -1)
		return false;
	
	res = twi_write(KTD_TWI_CH,
                             KTD_DEVICE_ADDR,
                             IGRN1,g);	
	if(res == -1)
		return false;
	
	res = twi_write(KTD_TWI_CH,
                             KTD_DEVICE_ADDR,
                             IBLU1,b);
	if(res == -1)
		return false;
	
	return true;	
}

/* color
	color count 8
	RED_SET0, GRN_SET0, BLU_SET0  -> 0
	RED_SET0, GRN_SET0, BLU_SET1  -> 1
	RED_SET0, GRN_SET1, BLU_SET0  -> 2
	RED_SET0, GRN_SET1, BLU_SET1  -> 3
	RED_SET1, GRN_SET0, BLU_SET0  -> 4
	RED_SET1, GRN_SET0, BLU_SET1  -> 5
	RED_SET1, GRN_SET1, BLU_SET0  -> 6
	RED_SET1, GRN_SET1, BLU_SET1  -> 7
*/

/*
#define LED1 	0x1
#define LED2 	0x2
#define LED3 	0x4
#define LED4 	0x8
*/

bool ktd_led_color(U16 sel_a, U16 sel_b, U16 sel_c, U8 color)
{			
	if(sel_a)
	{				
		if(sel_a & LED1)
		{
			isel_a12 &= ~(0x7  << 4);
			isel_a12 |=  color << 4;
		}
		
		if(sel_a & LED2)
		{
			isel_a12 &= ~(0x7 << 0);
            isel_a12 |= color << 0;
		}
		
		if(sel_a & LED3)
		{
			isel_a34 &= ~( 0x7 << 4);
			isel_a34 |=  color << 4;
		}
		
		if(sel_a & LED4)
		{
			isel_a34 &= ~(0x7 << 0);
            isel_a34 |= color << 0;		
		}
		
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELA12, isel_a12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELA34, isel_a34);		
	}
	
	if(sel_b)
	{
		if(sel_b & LED1)
		{
			isel_b12 &= ~(0x7 << 4);
            isel_b12 |= color << 4;
		}
		
		if(sel_b & LED2)
		{
			isel_b12 &= ~(0x7 << 0);
            isel_b12 |= color << 0;
		}
		
		if(sel_b & LED3)
		{
			isel_b34 &= ~(0x7 << 4);
            isel_b34 |= color << 4;
		}
			
		if(sel_b & LED4)
		{
			isel_b34 &= ~(0x7 << 0);
            isel_b34 |= color << 0;			
		}
		
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELB12, isel_b12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELB34, isel_b34);		
	}
	
	if(sel_c)
	{
		if(sel_c & LED1)
		{
			isel_c12 &= ~(0x7 << 4);
            isel_c12 |= color << 4;
		}
		
		if(sel_c & LED2)
		{
			isel_c12 &= ~(0x7 << 0);
            isel_c12 |= color << 0;
		}
		
		if(sel_c & LED3)
		{
			isel_c34 &= ~(0x7 << 4);
            isel_c34 |= color << 4;
		}
			
		if(sel_c & LED4)
		{
			isel_c34 &= ~(0x7 << 0);
            isel_c34 |= color << 0;			
		}
		
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELC12, isel_c12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELC34, isel_c34);		
	}	
	
	return true;
}

bool ktd_led_on(U16 sel_a, U16 sel_b, U16 sel_c, U8 b)
{	
	if(sel_a)
	{
		if(b)
		{
			if(sel_a & LED1)
				isel_a12 |= ISELA_ENA1;
			
			if(sel_a & LED2)
				isel_a12 |= ISELA_ENA2;
				
			if(sel_a & LED3)
				isel_a34 |= ISELA_ENA3;
				
			if(sel_a & LED4)
				isel_a34 |= ISELA_ENA4;			
		}
		else
		{
			if(sel_a & LED1)
				isel_a12 &= ~(ISELA_ENA1);
				
			if(sel_a & LED2)
				isel_a12 &= ~(ISELA_ENA2);
				
			if(sel_a & LED3)
				isel_a34 &= ~(ISELA_ENA3);
				
			if(sel_a & LED4)
				isel_a34 &= ~(ISELA_ENA4);			
		}		
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELA12, isel_a12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELA34, isel_a34);		
	}
	
	if(sel_b)
	{
		if(b)
		{
			if(sel_b & LED1)
				isel_b12 |= ISELB_ENB1;
			
			if(sel_b & LED2)
				isel_b12 |= ISELB_ENB2;
			
			if(sel_b & LED3)
				isel_b34 |= ISELB_ENB3;
			
			if(sel_b & LED4)
				isel_b34 |= ISELB_ENB4;			
		}
		else
		{
			if(sel_b & LED1)
				isel_b12 &= ~(ISELB_ENB1);
			
			if(sel_b & LED2)
				isel_b12 &= ~(ISELB_ENB2);
			
			if(sel_b & LED3)
				isel_b34 &= ~(ISELB_ENB3);
			
			if(sel_b & LED4)
				isel_b34 &= ~(ISELB_ENB4);			
		}
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELB12, isel_b12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELB34, isel_b34);		
	}
	
	if(sel_c)
	{
		if(b)
		{
			if(sel_c & LED1)
				isel_c12 |= ISELC_ENC1;
			
			if(sel_c & LED2)
				isel_c12 |= ISELC_ENC2;
			
			if(sel_c & LED3)
				isel_c34 |= ISELC_ENC3;
			
			if(sel_c & LED4)
				isel_c34 |= ISELC_ENC4;			
		}
		else
		{
			if(sel_c & LED1)
				isel_c12 &= ~(ISELC_ENC1);
			
			if(sel_c & LED2)
				isel_c12 &= ~(ISELC_ENC2);
			
			if(sel_c & LED3)
				isel_c34 &= ~(ISELC_ENC3);
			
			if(sel_c & LED4)
				isel_c34 &= ~(ISELC_ENC4);			
		}
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELC12, isel_c12);		
					
		twi_write(KTD_TWI_CH, 
                        KTD_DEVICE_ADDR, 
                        ISELC34, isel_c34);		
	}
	return true;
}

bool ktd_write(U8 addr, U8 data)
{	
	char res = 0;
	res = twi_write(KTD_TWI_CH, KTD_DEVICE_ADDR, addr, data);
	if(res == -1)
	{
		debugprintf("ktd init fail. \r\n");
		return false;
	}
	
	return true;
}

U8 ktd_read(U8 addr)
{	
	char res = 0;
	U8 data = 0;
	res = twi_read(KTD_TWI_CH, KTD_DEVICE_ADDR, &addr, 1, &data, 1);
	if(res == -1)
	{
		debugprintf("ktd init fail. \r\n");
		return false;
	}
	
	return true;
}

bool ktd_init()
{
	U8 addr = 0;
	U8 data = 0;
	char res = 0;
	twi_set_freq(0,100000);
	res = twi_read(KTD_TWI_CH, KTD_DEVICE_ADDR, &addr, 1, &data, 1);
	if(res == -1)
	{
		debugprintf("ktd init fail. \r\n");
		return false;
	}
	
	ktd_set_rgb0(0, 0, 0);		
	ktd_set_rgb1(0x30, 0x30, 0x30);
	
	debugprintf("ktd init complete. \r\n");	
	return true;
}
