#include "sdk.h"
#include "KTD2061.h"
#include "cm802_5.h"

extern void evmboardinit();
INCBIN(wav_demo1, "1.wav");  // Add demo1.wav file data, 
INCBIN(wav_demo2, "2.wav");  // Add demo2.wav file data, 
INCBIN(wav_demo3, "3.wav");  // Add demo3.wav file data, 
INCBIN(wav_demo4, "4.wav");  // Add demo4.wav file data, 

void demo1();
void demo2();
void demo3();
void demo4();	

int mode = 1;
int old_mode = 1;
int value = 1;
int intput_data = 0;

int first_in = 1;

int mode1_menu = 1;       
int mode3_menu = 1;
int mode3_delay = 100;
int mode4_menu = 100;

int main()
{
	evmboardinit();
	uart_config(5, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE);
	debugstring("================================================\r\n");  
	debugprintf("adLuna Demo. System Clock(%dMhz)\r\n", get_ahb_clock() / 1000000);
	debugstring("================================================\r\n");

	*R_GPIDIR(4) = 1<<3 | 1<<2 | 1<<1 | 1<<0;  // P4.0, P4.1, P4.2, P4.3 input
	*R_GPIDIR(0) = 1<<7 ;  // P0.7 input
	sound_init();	

	//sound_vol(235);

	WAVE* demo_wav[4];
	
	demo_wav[0] = sound_loadwavp((U8*)wav_demo1_data, wav_demo1_end - wav_demo1_data);
	demo_wav[1] = sound_loadwavp((U8*)wav_demo2_data, wav_demo2_end - wav_demo2_data);
	demo_wav[2] = sound_loadwavp((U8*)wav_demo3_data, wav_demo3_end - wav_demo3_data);
	demo_wav[3] = sound_loadwavp((U8*)wav_demo4_data, wav_demo4_end - wav_demo4_data);
	
	// lcd initial
	cm802_5_set();  
	cm802_5_init();	
	cm802_5_putstring(0,0 ,"DEMO 1");
	cm802_5_putstring(0,1 ,"        ");
	
	// RGB LED IC Init
	ktd_reset();  
	ktd_init();
	ktd_enable(CON_NIGHT_MODE);
	ktd_fade_rate(CON_FADE_500);
	ktd_led_on(LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, 1);
	
	sound_play(demo_wav[0]);
	
	while(1)
	{
		intput_data = *R_GPILEV(4) & 0xf;		
		
		switch(intput_data)
		{
			case 0x7: // right	
				if(value)
				{
					switch(mode)
					{
						case 1: // Demo 1, change color
							mode1_menu++;
							if(mode1_menu>7)
								mode1_menu = 1;						
							break;
						case 2: // Demo 2, change mode
							ktd_enable(CON_NOMAL_MODE);			
							cm802_5_putstring(0,1 ,"NOMAL");
							break;
						case 3: // Demo3, change dim
							mode3_menu++;
							if(mode3_menu>2)
								mode3_menu = 2;						
							break;
						case 4: // Demo4, change speed
							mode4_menu-=50;
							if(mode4_menu < 50)
								mode4_menu = 50;								
							break;
					}
					value = 0;
				}
				break;				
			case 0xB: // down
				if(value)
				{
					mode++;										
					if(mode > 4)
						mode = 4;													
					sound_stop(demo_wav[old_mode-1]);
					sound_play(demo_wav[mode-1]);
					old_mode = mode;										
					value = 0;
					first_in = 1;
				}				
				break;				
			case 0xD: // left				
				if(value)
				{
					switch(mode)
					{
						case 1: // Demo 1, change color
							mode1_menu--;
							if(mode1_menu<1)
								mode1_menu = 7;						
							break;
						case 2:  // Demo 2, change mode
							ktd_enable(CON_NIGHT_MODE);						
							cm802_5_putstring(0,1 ,"NIGHT");
							break;
						case 3: // Demo3, change dim
							mode3_menu--;
							if(mode3_menu<0)
								mode3_menu = 0;						
							break;
						case 4: // Demo4, change speed
							mode4_menu+=50;
							if(mode4_menu > 300)
								mode4_menu = 300;								
							break;
					}				
					value = 0;
				}
				break;				
			case 0xE: // up
				if(value)
				{
					mode--;
					value = 0;
					
					if(mode < 1)
						mode = 1;			
					
					sound_stop(demo_wav[old_mode-1]);
					sound_play(demo_wav[mode-1]);
					old_mode = mode;
					first_in = 1;
				}							
				break;				
			case 0xF: // default
				value = 1;
				break;
		}		
		
		switch(mode)
		{
			case 1:
				demo1();
				delayms(200);
				break;
			case 2:
				demo2();
				delayms(200);
				break;
			case 3:								
				demo3();
				delayms(mode3_delay);
				break;
			case 4:
				demo4();
				delayms(mode4_menu);				
				break;				
		}
	}
		
	while(1);
	return 0;
}


U32 get_adc(U8 ch)
{
    U32 adc_res=0;	
	*R_ADCCTRL2 = 1;
    *R_ADCCTRL = ch << 5 | F_ADCCTRL_APB128 | F_ADCCTRL_EN | F_ADCCTRL_STC;	    

    while(!(*R_ADCSTAT&F_ADCSTAT_ADCDR));
    *R_ADCCTRL = 0;
	
    adc_res = *R_ADCDATA&0xfff;    
	
    return adc_res;
}

void demo1()
{
	U32 adc_dat = get_adc(1);;
	
	if(first_in)
	{
		cm802_5_putstring(0,0 ,"DEMO 1");				
		cm802_5_putstring(0,1 ,"BRI-MID ");
		ktd_enable(CON_NOMAL_MODE);
		ktd_set_rgb1(0xF,0xF,0xF);
		ktd_fade_rate(CON_FADE_63);
		first_in = 0;
	}	
	
	if(adc_dat > 0xa00) 
	{
		ktd_set_rgb1(0x30,0x30,0x30);
		cm802_5_putstring(0,1 ,"BRI-High");
	}
	else if(adc_dat > 0x300)
	{
		ktd_set_rgb1(0xF,0xF,0xF);		
		cm802_5_putstring(0,1 ,"BRI-MID ");
	}
	else
	{
		ktd_set_rgb1(0x3,0x3,0x3);
		cm802_5_putstring(0,1 ,"BRI-LOW ");
	}
	
	ktd_led_color(LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, mode1_menu);						
}

U8 tmp = 0;
U8 color1 = 4;
U8 color2 = 6;
U8 color3 = 2;
U8 color4 = 3;
U8 color5 = 1;
U8 color6 = 5;
U8 color7 = 7;

void demo2()
{
	if(first_in)
	{
		cm802_5_putstring(0,0 ,"DEMO 2");			
		cm802_5_putstring(0,1 ,"NIGHT   ");
		ktd_set_rgb1(0x30,0x30,0x30);
		ktd_enable(CON_NIGHT_MODE);						
		ktd_fade_rate(CON_FADE_63);
		first_in = 0;
	}
	
	ktd_led_color(LED1, LED4, 0, 		color1); // Red
	ktd_led_color(LED2, 0, 		  LED1, color2); // Yellow
	ktd_led_color(LED3, 0, 		  LED2, color3); // Green
	ktd_led_color(LED4, 0, 		  LED3, color4); // Sky_Blue
	ktd_led_color(0, 		LED1, LED4,  color5); // Blue
	ktd_led_color(0, 		LED2, 0, 		color6); // Purple
	ktd_led_color(0, 		LED3, 0, 		color7); // White	
	
	tmp = color1;
	color1 = color2;
	color2 = color3;
	color3 = color4;
	color4 = color5;
	color5 = color6;
	color6 = color7;
	color7 = tmp;	
}

int mode3_flag = 0;
int mode3_value = 0;
U8 color_r[10] = { 0xC0,	0xC0, 	0xC0,	0x0,		0x0,		0x10, 	0xC0, 	0x0,		0x0,	    0xC0};
U8 color_g[10] = { 0x0,  	0x10,	0xC0,	0xC0,	0x0,		0x0,		0x0,		0xC0,	0x40,	0xC0};
U8 color_b[10] = { 0x0, 	0x00, 	0x00,	0x0,		0xC0,	0x70,	0xC0,	0x40,	0x10,	0xC0};

void demo3()
{
	if(first_in)
	{
		cm802_5_putstring(0,0 ,"DEMO 3");		
		cm802_5_putstring(0,1 ,"DIM-MID ");
		ktd_enable(CON_NIGHT_MODE);
		ktd_fade_rate(CON_FADE_250);
		ktd_set_rgb1(0,0,0);
		ktd_led_color(LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, LED1|LED2|LED3|LED4, 7);								
		first_in = 0;
	}
	
	if(mode3_menu == 2)
	{
		cm802_5_putstring(0,1 ,"DIM-UP  ");
		ktd_fade_rate(CON_FADE_125);
		mode3_delay = 50;
	}
	else if(mode3_menu == 1)
	{
		cm802_5_putstring(0,1 ,"DIM-MID ");
		ktd_fade_rate(CON_FADE_250);
		mode3_delay = 100;
	}
	else
	{
		cm802_5_putstring(0,1 ,"DIM-DOWN");
		ktd_fade_rate(CON_FADE_500);
		mode3_delay = 200;
	}	
	
	if(mode3_flag == 10)
	{
		ktd_set_rgb1(color_r[mode3_value], color_g[mode3_value], color_b[mode3_value]);
	}
	else if(mode3_flag == 20)
	{
		ktd_set_rgb1(0, 0, 0);
	}
	
	mode3_flag++;
	
	if(mode3_flag > 20)
	{
		mode3_value++;
		if(mode3_value>9)
			mode3_value = 0;				
		mode3_flag = 0;
	}	
}

int mode4_value = 0;
int color = 4;	

void demo4()
{
	if(first_in)
	{
		cm802_5_putstring(0,0 ,"DEMO 4");
		cm802_5_putstring(0,1 ,"SPD-100 ");
		ktd_set_rgb1(0x30,0x30,0x30);
		ktd_enable(CON_NIGHT_MODE);
		ktd_fade_rate(CON_FADE_63);					
		mode4_menu = 100;
		first_in = 0;
	}
	
	static U8 k_led_1[12] = {LED1, LED2, LED3, LED4, 0, 0, 0, 0, 0, 0, 0, 0};
	static U8 k_led_2[12] = {0, 0, 0, 0, LED1, LED2, LED3, LED4, 0, 0, 0, 0};
	static U8 k_led_3[12] = {0, 0, 0, 0, 0, 0, 0, 0, LED1, LED2, LED3, LED4};
	
	switch(mode4_menu)
	{
		case 300:
			cm802_5_putstring(0,1 ,"SPD-1   ");
			break;
		case 250:
			cm802_5_putstring(0,1 ,"SPD-2   ");
			break;	
		case 200:
			cm802_5_putstring(0,1 ,"SPD-3   ");
			break;
		case 150:
			cm802_5_putstring(0,1 ,"SPD-4   ");
			break;
		case 100:
			cm802_5_putstring(0,1 ,"SPD-5   ");
			break;
		case 50:
			cm802_5_putstring(0,1 ,"SPD-6   ");
			break;
	}
	
	if(mode4_value<12)
	{
		ktd_led_color(k_led_1[mode4_value], k_led_2[mode4_value], k_led_3[mode4_value],color);
	}
	else
	{
		ktd_led_color(k_led_1[mode4_value-12], k_led_2[mode4_value-12], k_led_3[mode4_value-12], 0);
	}
	
	mode4_value++;
	
	if(mode4_value>24)
	{
		color++;
		if(color>7)
			color = 1;
		mode4_value = 0;
	}	
}
