#include "sdk.h"

extern void evmboardinit();
 
void print_val(unsigned char * string, unsigned char* buf, int len)
{
	int i = 0;
	debugstring(string);
	debugprintf("\r\n");
	for(i=0;i<len;i++)
		debugprintf("%x ",buf[i]);
	
	debugprintf("\r\n");	
}

unsigned char pps[4] = {0xff, 0x10, 0x13, 0xfc}; // 38400
//unsigned char pps[4] = {0xff, 0x10, 0x18, 0xf7}; // 115200
unsigned char sel_pse[5] = {0, 0xA4, 0x04, 0, 0x0E};
unsigned char get_lot[5] = {0, 0xCA, 0x01, 0x1, 0x8};
unsigned char pse_ds_req[14] = {0x31, 0x50, 0x41, 0x59, 0x2e, 0x53, 0x59, 0x53, 0x2e, 0x44, 0x44, 0x46, 0x30, 0x31};
unsigned char pse_data_req[5] = {0, 0xC0, 0, 0, 0x22};
unsigned char sel_df[5] = {0, 0xA4, 0x04, 0, 0x07};
unsigned char df_ds_req[7] = { 0xA0, 0, 0, 0x02, 0x45, 0, 0x1};
unsigned char df_data_req[5] = { 0, 0xc0, 0, 0, 0x1a};
    
unsigned char res[63];

#define SC_CH	3	

void sci_init(unsigned char ch, unsigned int cycle);
void sci_send_command(unsigned ch, unsigned char* cmd, unsigned char num);
unsigned int sci_read_res(unsigned char ch, unsigned char* res, unsigned char num);

U8 card_status = 1;
U8 old_card_status = 0;

void GPIO_ISR()
{
	if(*R_GPILEV(8) & (1<<6)) // Card out
	{
		*R_GPOHIGH(8) = 1<<5;  // PWR Off
		*R_GPOHIGH(8) = 1<<7;  // Reset
		cm802_5_putstring(0,0 ,"No Card");
		cm802_5_putstring(0,1 ,"        ");
		card_status = 0;
	}
	else // Card In
	{
		*R_GPOLOW(8) = 1<<5;  // PWR On
		*R_GPOLOW(8) = 1<<7;  // Reset clear
		cm802_5_putstring(0,0 ,"Card IN");	
		cm802_5_putstring(0,1 ,"        ");
		card_status = 1;
	}	
}

int main()
{
	evmboardinit();
	uart_config(5, 115200, DATABITS_8, STOPBITS_1, UART_PARNONE);
	debugstring("================================================\r\n");  
	debugprintf("adLuna ISO 7816 . System Clock(%dMhz)\r\n", get_ahb_clock() / 1000000);
	debugstring("================================================\r\n");
	
	// spi_lcd initial
	cm802_5_set(); 
	cm802_5_init();
	cm802_5_putstring(0,0 ,"        ");
	cm802_5_putstring(0,1 ,"        ");
	
	*R_PAF(0) &=  ~(F_PAF0_3_GP3 | F_PAF0_4_GP4);
	*R_PAF(0) |=  F_PAF0_3_UART3_RX | F_PAF0_4_UART3_TX;          // UART 3
	*R_PAF(8) |= F_PAF8_5_GP5 | F_PAF8_6_GP6 | F_PAF8_7_GP7;   // GPIO 8.7 CARD Rst, GPIO 8.5 CARD Pwr, GPIO 8.6 CD 1
	
	*R_GPOHIGH(8) = 1<<5;  // PWR Off
	*R_GPOHIGH(8) = 1<<7;  // Reset
	
	*R_GPIDIR(8) = 1<<6; // GPIO 8.6 CARD check -> input
	*R_GPODIR(8) = 1<<7 | 1<<5; // GPIO 8.7 CARD Rst, GPIO 8.5 CARD Pwr -> output
	
	*R_GPIRQMODE(8) = 0x3; // Falling/Rising interrupt Mode
	*R_GPIRQEN(8) = 1<<6; // P8.6 Interrupt enable
	set_interrupt(INTNUM_GPIO8, GPIO_ISR);	
	enable_interrupt(INTNUM_GPIO8, true);		
		
	while(1)
	{
		if((*R_GPILEV(8) & (1<<6))==0)
		{
			card_status = 1;
			if(card_status != old_card_status)
			{				
				debugprintf("card in\r\n");
				cm802_5_putstring(0,0 ,"Card IN");			
				
				*R_GPOLOW(8) = 1<<5;  // PWR On
				*R_GPOLOW(8) = 1<<7;  // Reset clear
				break;
			}
		}
		else
		{
			if(card_status != old_card_status)
			{
				card_status = 0;
				debugprintf("no card\r\n");			
				cm802_5_putstring(0,0 ,"No Card");				
			}			
		}				
	}	
	
	while(1)
	{
		if(card_status != old_card_status)
		{
			if(card_status == 1)
			{										
				sci_init(SC_CH, 372);
				debugprintf("ATR Wait ... \r\n");
				cm802_5_putstring(0,1 ,"ATR Wait");
			
				*R_GPOHIGH(8) = 1<<7; // Reset High			
				delayms(200);
			
				sci_read_res(SC_CH, res, 12);			
				debugprintf("ATR Data Done \r\n");	
				cm802_5_putstring(0,1 ,"ATR Done");
				print_val("ATR Data", res, 12);			
			
				sci_send_command(SC_CH, pps, 4);		
				sci_read_res(SC_CH, res, 4);
					
				print_val("command", pps, 4);		
				print_val("response", res, 4);
				debugstring("\r\n");	
				
				// Baudrate change
				debugprintf("Baudrate change \r\n");	
				sci_init(SC_CH,372/4);
				delayms(100);	
				
				// Select PSE
				debugprintf("Select PSE \r\n");	
				cm802_5_putstring(0,1 ,"Sel PSE ");
				sci_send_command(SC_CH, sel_pse, 5);				
				sci_read_res(SC_CH, res, 1);
					
				print_val("command", sel_pse, 5);	
				print_val("response", res, 1);	
				debugstring("\r\n");	
				
				// PSE Size
				debugprintf("PSE Size Request \r\n");	
				cm802_5_putstring(0,1 ,"PSE Size");
				sci_send_command(SC_CH, pse_ds_req, 14);		
				sci_read_res(SC_CH, res, 2);
				
				print_val("command", pse_ds_req, 14);	
				print_val("response", res, 2);		
				debugstring("\r\n");	
				
				// PSE Data
				debugprintf("PSE Data Request \r\n");	
				cm802_5_putstring(0,1 ,"PSE Data");
				sci_send_command(SC_CH, pse_data_req, 5);			
				sci_read_res(SC_CH, res, 37);
					
				print_val("command", pse_data_req, 5);	
				print_val("response", res, 37);		
				debugstring("\r\n");	
				
				// Select DF
				debugprintf("Select DF\r\n");	
				cm802_5_putstring(0,1 ,"Sel DF  ");
				sci_send_command(SC_CH, sel_df, 5);		
				sci_read_res(SC_CH, res, 1);
				
				print_val("command", sel_df, 5);	
				print_val("response", res, 1);		
				debugstring("\r\n");	
				
				// DF Size
				debugprintf("DF Size Request \r\n");	
				cm802_5_putstring(0,1 ,"DF Size ");
				sci_send_command(SC_CH, df_ds_req, 7);		
				sci_read_res(SC_CH, res, 2);
					
				print_val("command", df_ds_req, 7);	
				print_val("response", res, 2);		
				debugstring("\r\n");	
				
				// DF Data
				debugprintf("DF Data Request \r\n");	
				cm802_5_putstring(0,1 ,"DF Data ");
				sci_send_command(SC_CH, df_data_req, 5);			
				sci_read_res(SC_CH, res, 0x1a+3);
					
				print_val("command", df_data_req, 5);	
				print_val("response", res, 0x1A+3);	
				debugstring("\r\n");	
				
				debugprintf("Test End.\r\n");	
				cm802_5_putstring(0,1 ,"Test END");
			}
			old_card_status = card_status;			
		}
	}	
	while(1);
	return 0;
}

//#define SCI_DEBUG

void sci_init(unsigned char ch, unsigned int cycle)
{
	volatile U32 uart_base = (U32)R_UART_BASE(ch);	
	unsigned int baud_tmp;
	
	uart_config(ch,9600,DATABITS_8,STOPBITS_1,UART_PAREVEN); 
	
	// set baud rate 
	/************************************************************/
	*(volatile U32 *)(uart_base + UART_LCR) |= F_ULC_DLAB;		
	baud_tmp = cycle * 4;	
	*(volatile U32 *)(uart_base + UART_FDR) = baud_tmp & 0x3F;
	baud_tmp = baud_tmp >> 6;	
	*(volatile U32 *)(uart_base + UART_DIV_LO) = baud_tmp & 0xff;	
	*(volatile U32 *)(uart_base + UART_DIV_HI) = (baud_tmp >> 8) & 0xff;		
	/************************************************************/

	*(volatile U32 *)(uart_base + UART_LCR) &= ~(F_ULC_DLAB);		
	*(volatile U32 *)(uart_base + UART_SCI) = SCI_CLK_EN | SCI_RX_EN | SCI_PCLK_4;   // 1/4 divider	
	//*(volatile U32 *)(uart_base + UART_SCI) = SCI_CLK_EN | SCI_RX_EN | SCI_PCLK_2;   // 1/2 divider					
	*(volatile U32 *)(uart_base + UART_IFC) = 1<<0;   // ISO7816		
}

void sci_send_command(unsigned ch, unsigned char* cmd, unsigned char num)
{
	volatile U32 uart_base = (U32)R_UART_BASE(ch);	
	unsigned char i;	
	
	*(volatile U32 *)(uart_base + UART_SCI) &= 0xDF; 			// Rx disable;
	*(volatile U32 *)(uart_base + UART_SCI) |= SCI_TX_EN; // Tx enable;	
	
	for(i=0; i<num; i++ )
	{		
		delayus(100);
		uart_putch(ch, cmd[i]);
		while (!((*(volatile U32 *)(uart_base + UART_LSR))  & F_ULS_TEMP));				
	}		
	*(volatile U32 *)(uart_base + UART_SCI) &= 0xEF; 			// Tx disable;
	*(volatile U32 *)(uart_base + UART_SCI) |= SCI_RX_EN; // Rx enable;	
	delayms(10);	
}

unsigned int sci_read_res(unsigned char ch, unsigned char* res, unsigned char num)
{	
	unsigned int i = 0;
	unsigned int tmp_cnt = 0;
	unsigned char tmp_buf[128];
	
	for(i=0;i<num;i++)
	{
		if(uart_getch(ch,&res[i]) == 0) // if getch_fail, retry
			i--;		
	}	
	
	tmp_cnt = uart_getdata(ch, tmp_buf, 128);  // remain data flush.
	
#ifdef SCI_DEBUG
	if(tmp_cnt)
	{
		debugprintf("\r\nRemain data \r\n");
		for(i=0;i<tmp_cnt;i++)
		{
			debugprintf("%x  ", tmp_buf[i]);
		}
		debugprintf("\r\n");
	}	
#endif
	return num;
}
