
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include <strsafe.h>
//for USBUSB
#include <winusb.h>
#include <Usb100.h>
#include <Setupapi.h>

#pragma comment(lib,"winusb.lib")
#pragma comment(lib,"setupapi.lib") // install WDK

#include "AdcBulkUsb.h"



CAdcBulkUsb::CAdcBulkUsb(void)
{
	m_UsbDevInfo.hDeviceHandle = INVALID_HANDLE_VALUE;
	m_UsbDevInfo.winUSBHandle = INVALID_HANDLE_VALUE;
	//m_hDevice = INVALID_HANDLE_VALUE;
	//ZeroMemory(&m_UsbDevInfo,sizeof(m_UsbDevInfo));
}


CAdcBulkUsb::~CAdcBulkUsb(void)
{
}



void PrintLastError(LPTSTR lpszFunction)
{
	LPVOID lpMsgBuf;
	DWORD dw = GetLastError(); 

	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		dw,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR) &lpMsgBuf,
		0, NULL );
	/*
	printf("%s failed with error %d: %s", 
		lpszFunction, dw, lpMsgBuf); 
		*/
	LocalFree(lpMsgBuf);

}
BOOL GetDeviceHandle (GUID guidDeviceInterface, PHANDLE hDeviceHandle,  int deviceNum)
{
	if (guidDeviceInterface==GUID_NULL)
	{
		return FALSE;
	}

	BOOL bResult = TRUE;
	HDEVINFO hDeviceInfo;
	SP_DEVINFO_DATA DeviceInfoData;

	SP_DEVICE_INTERFACE_DATA deviceInterfaceData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA pInterfaceDetailData = NULL;

	ULONG requiredLength=0;

	LPTSTR lpDevicePath = NULL;

	DWORD index = 0;

	// Get information about all the installed devices for the specified
	// device interface class.
	hDeviceInfo = SetupDiGetClassDevs( 
		&guidDeviceInterface,
		NULL, 
		NULL,
		DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);

	if (hDeviceInfo == INVALID_HANDLE_VALUE) 
	{ 
		// ERROR 
		PrintLastError("Error SetupDiGetClassDevs:");
		goto done;
	}

	//Enumerate all the device interfaces in the device information set.
	DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);

	for (index = deviceNum; SetupDiEnumDeviceInfo(hDeviceInfo, index, &DeviceInfoData); index++)
	{
		//Reset for this iteration
		if (lpDevicePath)
		{
			LocalFree(lpDevicePath);
		}
		if (pInterfaceDetailData)
		{
			LocalFree(pInterfaceDetailData);
		}

		deviceInterfaceData.cbSize = sizeof(SP_INTERFACE_DEVICE_DATA);

		//Get information about the device interface.
		bResult = SetupDiEnumDeviceInterfaces( 
			hDeviceInfo,
			&DeviceInfoData,
			&guidDeviceInterface,
			0, 
			&deviceInterfaceData);

		// Check if last item
		if (GetLastError () == ERROR_NO_MORE_ITEMS)
		{
			break;
		}

		//Check for some other error
		if (!bResult) 
		{
			PrintLastError("Error SetupDiEnumDeviceInterfaces:");
			goto done;
		}

		//Interface data is returned in SP_DEVICE_INTERFACE_DETAIL_DATA
		//which we need to allocate, so we have to call this function twice.
		//First to get the size so that we know how much to allocate
		//Second, the actual call with the allocated buffer

		bResult = SetupDiGetDeviceInterfaceDetail(
			hDeviceInfo,
			&deviceInterfaceData,
			NULL, 0,
			&requiredLength,
			NULL);


		//Check for some other error
		if (!bResult) 
		{
			if ((ERROR_INSUFFICIENT_BUFFER==GetLastError()) && (requiredLength>0))
			{
				//we got the size, allocate buffer
				pInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)LocalAlloc(LPTR, requiredLength);

				if (!pInterfaceDetailData) 
				{ 
					// ERROR 
					printf("Error allocating memory for the device detail buffer.\n");
					goto done;
				}
			}
			else
			{
				printf("Error SetupDiEnumDeviceInterfaces: %d.\n", GetLastError());
				goto done;
			}
		}

		//get the interface detailed data
		pInterfaceDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

		//Now call it with the correct size and allocated buffer
		bResult = SetupDiGetDeviceInterfaceDetail(
			hDeviceInfo,
			&deviceInterfaceData,
			pInterfaceDetailData,
			requiredLength,
			NULL,
			&DeviceInfoData);

		//Check for some other error
		if (!bResult) 
		{
			PrintLastError("Error SetupDiGetDeviceInterfaceDetail: ");
			goto done;
		}

		//copy device path

		size_t nLength = strlen (pInterfaceDetailData->DevicePath) + 1;  
		lpDevicePath = (TCHAR *) LocalAlloc (LPTR, nLength * sizeof(TCHAR));
		StringCchCopy(lpDevicePath, nLength, pInterfaceDetailData->DevicePath);
		lpDevicePath[nLength-1] = 0;
		int st = SetupDiEnumDeviceInfo(hDeviceInfo, index, &DeviceInfoData);		

		if(lpDevicePath)
		{
			break;
		}	
		//printf("Device path:  %s\n", lpDevicePath);
	}

	if (!lpDevicePath)
	{
		//Error.
		//PrintLastError("Error :");
		goto done;
	}

	//Open the device
	*hDeviceHandle = CreateFile (
		lpDevicePath,
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,
		NULL);

	if (*hDeviceHandle == INVALID_HANDLE_VALUE)
	{
		printf("create fail\n");
		//Error.
		PrintLastError("Error ");
		goto done;
	}



done:
	LocalFree(lpDevicePath);
	LocalFree(pInterfaceDetailData);    
	bResult = SetupDiDestroyDeviceInfoList(hDeviceInfo);

	return bResult;
}
BOOL GetWinUSBHandle(HANDLE hDeviceHandle, PWINUSB_INTERFACE_HANDLE phWinUSBHandle)
{
	if (hDeviceHandle == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	BOOL bResult = WinUsb_Initialize(hDeviceHandle, phWinUSBHandle);
	if(!bResult)
	{
		//Error.
		printf("WinUsb_Initialize Error %d.", GetLastError());
		return FALSE;
	}

	return bResult;
}
BOOL GetUSBDeviceSpeed(WINUSB_INTERFACE_HANDLE hDeviceHandle, UCHAR* pDeviceSpeed)
{
	if (!pDeviceSpeed || hDeviceHandle==INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	BOOL bResult = TRUE;

	ULONG length = sizeof(UCHAR);

	bResult = WinUsb_QueryDeviceInformation(hDeviceHandle, DEVICE_SPEED, &length, pDeviceSpeed);
	if(!bResult)
	{
		printf("Error getting device speed: %d.\n", GetLastError());
		goto done;
	}

	if(*pDeviceSpeed == LowSpeed)
	{
		printf("Device speed: %d (Low speed).\n", *pDeviceSpeed);
		goto done;
	}
	if(*pDeviceSpeed == FullSpeed)
	{
		printf("Device speed: %d (Full speed).\n", *pDeviceSpeed);
		goto done;
	}
	if(*pDeviceSpeed == HighSpeed)
	{
		printf("Device speed: %d (High speed).\n", *pDeviceSpeed);
		goto done;
	}

done:
	return bResult;
}
struct PIPE_ID
{
	UCHAR  PipeInId;
	UCHAR  PipeOutId;
};

BOOL QueryDeviceEndpoints (WINUSB_INTERFACE_HANDLE hDeviceHandle, PIPE_ID* pipeid)
{
	if (hDeviceHandle==INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	BOOL bResult = TRUE;

	USB_INTERFACE_DESCRIPTOR InterfaceDescriptor;
	ZeroMemory(&InterfaceDescriptor, sizeof(USB_INTERFACE_DESCRIPTOR));

	WINUSB_PIPE_INFORMATION  Pipe;
	ZeroMemory(&Pipe, sizeof(WINUSB_PIPE_INFORMATION));


	bResult = WinUsb_QueryInterfaceSettings(hDeviceHandle, 0, &InterfaceDescriptor);

	if (bResult)
	{
		for (int index = 0; index < InterfaceDescriptor.bNumEndpoints; index++)
		{
			bResult = WinUsb_QueryPipe(hDeviceHandle, 0, index, &Pipe);

			if (bResult)
			{
				if (Pipe.PipeType == UsbdPipeTypeControl)
				{
					//printf("Endpoint index: %d Pipe type: Control Pipe ID: %d.\n", index, Pipe.PipeType, Pipe.PipeId);
				}
				if (Pipe.PipeType == UsbdPipeTypeIsochronous)
				{
					//printf("Endpoint index: %d Pipe type: Isochronous Pipe ID: %d.\n", index, Pipe.PipeType, Pipe.PipeId);
				}
				if (Pipe.PipeType == UsbdPipeTypeBulk)
				{
					if (USB_ENDPOINT_DIRECTION_IN(Pipe.PipeId))
					{
						//printf("Endpoint index: %d Pipe type: Bulk Pipe ID: %d.\n", index, Pipe.PipeType, Pipe.PipeId);
						pipeid->PipeInId = Pipe.PipeId;
					}
					if (USB_ENDPOINT_DIRECTION_OUT(Pipe.PipeId))
					{
						//printf("Endpoint index: %d Pipe type: Bulk Pipe ID: %d.\n", index, Pipe.PipeType, Pipe.PipeId);
						pipeid->PipeOutId = Pipe.PipeId;
					}

				}
				if (Pipe.PipeType == UsbdPipeTypeInterrupt)
				{
					//printf("Endpoint index: %d Pipe type: Interrupt Pipe ID: %d.\n", index, Pipe.PipeType, Pipe.PipeId);
				}
			}
			else
			{
				continue;
			}
		}
	}

	return bResult;
}

BOOL SendDatatoDefaultEndpoint(WINUSB_INTERFACE_HANDLE hDeviceHandle)
{
	if (hDeviceHandle==INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	BOOL bResult = TRUE;


	UCHAR bars = 0;

	WINUSB_SETUP_PACKET SetupPacket;
	ZeroMemory(&SetupPacket, sizeof(WINUSB_SETUP_PACKET));
	ULONG cbSent = 0;

	//Set bits to light alternate bars
	for (short i = 0; i < 7; i+= 2)
	{
		bars += 1 << i;
	}

	//Create the setup packet
	SetupPacket.RequestType = 0;
	SetupPacket.Request = 0xD8;
	SetupPacket.Value = 0;
	SetupPacket.Index = 0; 
	SetupPacket.Length = sizeof(UCHAR);

	bResult = WinUsb_ControlTransfer(hDeviceHandle, SetupPacket, &bars, sizeof(UCHAR), &cbSent, 0);
	if(!bResult)
	{
		goto done;
	}


done:
	return bResult;

}

BOOL CAdcBulkUsb::Open(LPGUID  pGuid ,int deviceNum/* = 0*/)

{
	GUID guidDeviceInterface = *pGuid; //in the INF file
	PIPE_ID PipeID;
	BOOL bResult = TRUE;
	HANDLE hDeviceHandle;
	WINUSB_INTERFACE_HANDLE hWinUSBHandle;


	ULONG cbSize = 0;
	hDeviceHandle = 0;
	bResult = GetDeviceHandle(guidDeviceInterface, &hDeviceHandle, deviceNum);
	if(!bResult)
	{
		return FALSE;
	}

	bResult = GetWinUSBHandle(hDeviceHandle, &hWinUSBHandle);
	if(!bResult)
	{
		if(hDeviceHandle!=NULL)
			CloseHandle(hDeviceHandle);
		return FALSE;
	}
	bResult = QueryDeviceEndpoints(hWinUSBHandle, &PipeID);
	if(!bResult)
	{
		CloseHandle(hDeviceHandle);
		WinUsb_Free(hWinUSBHandle);
		return FALSE;
	}
	m_UsbDevInfo.hDeviceHandle = hDeviceHandle;
	m_UsbDevInfo.winUSBHandle = hWinUSBHandle;
	m_UsbDevInfo.bulkInPipe = PipeID.PipeInId;
	m_UsbDevInfo.bulkOutPipe= PipeID.PipeOutId;

	DWORD dwMilliseconds = 10*1000;//10sec
	WinUsb_SetPipePolicy(hWinUSBHandle,PipeID.PipeInId,PIPE_TRANSFER_TIMEOUT,sizeof(DWORD),&dwMilliseconds);
	WinUsb_SetPipePolicy(hWinUSBHandle,PipeID.PipeOutId,PIPE_TRANSFER_TIMEOUT,sizeof(DWORD),&dwMilliseconds);

	return TRUE;
}

BOOL CAdcBulkUsb::Read(LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead,DWORD dwMilliseconds)
{
	if(m_UsbDevInfo.winUSBHandle == INVALID_HANDLE_VALUE)
	{
		*lpNumberOfBytesRead = 0;
		return FALSE;
	}
	BOOL Result;
	WinUsb_SetPipePolicy(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkInPipe,PIPE_TRANSFER_TIMEOUT,sizeof(DWORD),&dwMilliseconds);
	Result = WinUsb_ReadPipe(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkInPipe,(PUCHAR)lpBuffer,nNumberOfBytesToRead,lpNumberOfBytesRead,NULL);
	return Result;
}


BOOL CAdcBulkUsb::Write(LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten,DWORD dwMilliseconds)
{
	if(m_UsbDevInfo.winUSBHandle == INVALID_HANDLE_VALUE)
	{
		*lpNumberOfBytesWritten = 0;
		return FALSE;
	}
	BOOL Result;
	WinUsb_SetPipePolicy(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkOutPipe,PIPE_TRANSFER_TIMEOUT,sizeof(DWORD),&dwMilliseconds);
	Result = WinUsb_WritePipe(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkOutPipe,(PUCHAR)lpBuffer,nNumberOfBytesToWrite,lpNumberOfBytesWritten,NULL);
	return Result;
}


void CAdcBulkUsb::Close()
{
	if(m_UsbDevInfo.winUSBHandle != INVALID_HANDLE_VALUE)
	{
		WinUsb_Free(m_UsbDevInfo.winUSBHandle);
		m_UsbDevInfo.winUSBHandle = INVALID_HANDLE_VALUE;
	}
	if(m_UsbDevInfo.hDeviceHandle != INVALID_HANDLE_VALUE)
		CloseHandle(m_UsbDevInfo.hDeviceHandle );

}

BOOL CAdcBulkUsb::CancelRead()
{
	if(m_UsbDevInfo.winUSBHandle != INVALID_HANDLE_VALUE)
	{
		return WinUsb_AbortPipe(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkInPipe);
	}
	return FALSE;
}


BOOL CAdcBulkUsb::CancelWrite()
{
	if(m_UsbDevInfo.winUSBHandle != INVALID_HANDLE_VALUE)
	{
		return WinUsb_AbortPipe(m_UsbDevInfo.winUSBHandle,m_UsbDevInfo.bulkOutPipe);
	}
	return FALSE;
}