/******************************************************************************
 Copyright (C) 2011      Advanced Digital Chips Inc. 
						http://www.adc.co.kr
 Author : Software Team.

******************************************************************************/
#include "sdk.h"

extern void evmboardinit();

extern volatile U32 usb_data_out_ready;
extern volatile U32 usb_data_key;
extern volatile U32 usb_data_key_1;
extern volatile U32 usb_data_key_2;
extern volatile U32 usb_data_segment;

U8 usb_data_in_array[HID_INPUT_REPORT_BYTES];
extern volatile U8* usb_data_in_ptr;

U8 usb_data_out_array[HID_OUTPUT_REPORT_BYTES];
extern volatile U8* usb_data_out_ptr;

extern U8 USB_HID_Configuration_Descriptor[];
extern U8 USB_HID_Device_Descriptor[];
extern U8 HID_ReportDescriptor[];
extern U8 USB_StringDescriptor[];

extern U8 USB_String_Des_size;
extern U8 HID_Report_Des_size;

void usb_irq_f()
{
	BYTE ep_irq ;
    BYTE usb_irq ;	
	ep_irq  = *(volatile unsigned char*)__USBEI_REG;
	usb_irq = *(volatile unsigned char*)__USBUI_REG;
	
	if(ep_irq & EP0_IRQ)
	{		
		ep0_isr();
	}
	if (ep_irq & EP4_IRQ)
	{						
		ep4_isr();
	}

	if(usb_irq & USB_RESET_IRQ)
	{
		usb_reset();
	}
	else if ( usb_irq & USB_RESUME_IRQ )
	{
		usb_resume();
	}
	else if ( usb_irq & USB_SUSPEND_IRQ )
	{
		usb_suspend();
	}
	
	*(volatile U8*)__USBEI_REG = ep_irq;   // Clear Interrupt 
	*(volatile U8*)__USBUI_REG = usb_irq;  // Clear Interrupt
}

int main()
{
	evmboardinit();
	uart_config(5,115200,DATABITS_8,STOPBITS_1,UART_PARNONE);
	debugstring("START\r\n");
	debugstring("================================================\r\n");
	debugprintf("ADLUNA USB HID System Clock(%dMhz)\r\n",get_ahb_clock()/1000000);
	debugstring("================================================\r\n");

	usb_data_in_ptr = usb_data_in_array;
	usb_data_out_ptr = usb_data_out_array;
	
	set_interrupt(INTNUM_USB,usb_irq_f);
	enable_interrupt(INTNUM_USB,TRUE);

	int i,j;
	for(i=0,j=HID_INPUT_REPORT_BYTES;i<HID_INPUT_REPORT_BYTES;i++,j--)
	{
		usb_data_in_ptr[i] = j;
	}	
	usb_data_in_ptr[0] = 0;	
	
	usb_clock_init();
		
	usb_set_hid_description(USB_HID_Configuration_Descriptor,USB_HID_Device_Descriptor, HID_ReportDescriptor,USB_StringDescriptor);
    usb_init(USB_HID_MODE);	

	delayms(1000);
	
	U8 ch;
	U32 usb_data_out_state=0;
	
	//usb_data_out_ptr[]   PC -> EVM data
	//usb_data_in_ptr[]    PC <- EVM data
	
	while(1)
	{
		if(usb_data_out_ready == 1 && usb_data_out_state == 0)
		{
			usb_data_out_state = 1;		
		}		
		else if(usb_data_out_ready == 0 && usb_data_out_state == 1)
		{
			// print
			debugstring("\r\n");			
			debugprintf("USB IN Data[0] = %d / 0x%x\r\n",usb_data_out_ptr[0],usb_data_out_ptr[0]);
			/*for(i=0;i<HID_OUTPUT_REPORT_BYTES;i++)
			{
				debugprintf("USB OUT Data[%d] = %d / 0x%x\r\n",i,usb_data_out_ptr[i],usb_data_out_ptr[i]);
				// PRINTVAR(usb_data_out_ptr[i]);
			}*/
			
			usb_data_out_state = 0;
		}		
		
		if(uart_getch(5,&ch))
		{
			switch(ch)
			{
				case '1':					
					usb_data_key = 0x1;					
					break;
				case '2':					
					usb_data_key = 0x2;
					break;
				case '3':					
					usb_data_key = 0x4;
					break;
				case '4':					
					usb_data_key = 0x8;
					break;
				case '5':					
					usb_data_key = 0x10;
					break;
				case '6':					
					usb_data_key = 0x20;
					break;
				case '7':					
					usb_data_key = 0x40;
					break;
				case '8':					
					usb_data_key = 0x80;
					break;
				default:					
					usb_data_key = 0;					
			}
			
			usb_data_in_ptr[0] = usb_data_key;
			
			/*
			for(i=0;i<HID_OUTPUT_REPORT_BYTES;i++)
				usb_data_in_ptr[i] = usb_data_key;
			*/		
			
			debugstring("\r\n");			
			debugprintf("USB OUT Data[0] = %d / 0x%x\r\n",usb_data_key,usb_data_key);
		}		
	}
	
	return 0;

}

