/*****************************************************************************
*
* Copyright (C) 2014      Advanced Digital Chips, Inc. All Rights Reserved.
*						http://www.adc.co.kr
*
* THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
* EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE
*
*****************************************************************************/
#include "sdk.h"


enum {
PRE_SCALE_0=0,
PRE_SCALE_2,
PRE_SCALE_8,
PRE_SCALE_32,
PRE_SCALE_128,
PRE_SCALE_2K,
PRE_SCALE_8K,
PRE_SCALE_32K,
};

#define PRE_SCALE PRE_SCALE_32

/*
return nano seconds.
*/
static U32 nano = 0;
static U32 tick_op_freq = 0;
static inline bool update_freq()
{
	const int pre_val[] = { 2,2,8,32,128,2048,8192,32768 };
	int pres = ((*R_TTMCON) & 0xe) >> 1;
	pres = pre_val[pres];
	
	if (tick_op_freq == 0)
		tick_op_freq = get_ahb_clock()/ pres;
	else if (tick_op_freq != (get_ahb_clock()/ pres))
	{
		tick_op_freq = get_ahb_clock()/ pres;
		return true;
	}
	return false;
}

static inline U32 calc_one_tick()
{
	if(update_freq())
		nano = 1000000000 / tick_op_freq ;
	if(nano==0)
		nano = 1000000000 / tick_op_freq ;
	return nano;
}
static inline U32 tick_to_ms(int tickcnt)
{
	update_freq();
	return tickcnt / (tick_op_freq/1000);
}
static inline U32 tick_to_us(int tickcnt)
{
	update_freq();
	//return tickcnt * 1000000 / op_freq;
	return tickcnt / (tick_op_freq/1000000);
}
static inline U64 tick_to_ms64(U64 tickcnt)
{
	update_freq();
	return tickcnt * 1000 / (U64)tick_op_freq;
}
static inline U64 tick_to_us64(U64 tickcnt)
{
	update_freq();
	return tickcnt * 1000000 / (U64)tick_op_freq;
}


void tick_timer_init()
{
	*R_TTMRST = 2;//reset
	*R_TTMRST = 0;
	*R_TTMCNT1 = -1;
	*R_TTMCNT0 = -1;
	*R_TTMCON = (PRE_SCALE << 1) | 1;
	calc_one_tick(); //for update freq and nano variable.
}
/// <summary>
/// Retrieves the number of milliseconds that have elapsed since the system was started, up to 42 seconds if "Pre-scale Factor" is 2 and AHB is 200Mhz 
/// The return value is the number of milliseconds that have elapsed since the tick_timer_init() was started.
/// The elapsed time is stored as a 32bit value.Therefore, the time will wrap around to zero.
/// </summary>
/// <returns>milliseconds</returns>
U32 get_tick_count()
{
	U32 tickcnt = 0xffffffffUL - *R_TTMCNT0;
	return tick_to_ms(tickcnt);
}

/*
The return value is the number of milliseconds that have elapsed since the tick_timer_init() was started.
*/
U64 get_tick_count64()
{
	U64 tickcnt = 0;
	tickcnt = (U64)*R_TTMCNT0 + (((U64)(*R_TTMCNT1)) << 32);
	tickcnt = 0xffffffffffffffffULL - tickcnt;
	return tick_to_ms64(tickcnt);

}
static BOOL delayus_r(U32 us)
{
	U64 waitcnt = (U64)us * 1000 / calc_one_tick();
	U32 firstcnt = *R_TTMCNT0; //down count
	int n_wrapcnt = waitcnt > firstcnt ? 1 : 0;
	//need wrap?
	if (n_wrapcnt)
	{
		while (1)
		{
			if (firstcnt < *R_TTMCNT0)
				break;
		}
		waitcnt -= firstcnt;
	}
	U32 i;
	for(i=0;i<-1;i++)
	{
		if ((firstcnt - *R_TTMCNT0) > waitcnt)
		{
			return true;
		}
	}
	return FALSE;
}

BOOL delayus(U32 us)
{
	if (us > 20000000)
	{
		int i;
		int cnt = us / 20000000;
		for (i = 0; i < cnt; i++)
		{
			delayus_r(20000000);
		}
		us -= (cnt * 20000000);
	}
	return delayus_r(us);
}

BOOL delayms(U32 ms)
{
#if 0 
	if (ms > 20000)
	{
		int i;
		int cnt = ms / 20000;
		for (i = 0; i < cnt; i++)
		{
			delayus_r(20000 * 1000);
		}
		ms -= (cnt * 20000);
	}
	return delayus_r(ms * 1000);
#else
	U64 firstms = get_tick_count64();

	while (1)
	{
		if ((get_tick_count64() - firstms) > (U64)ms)
			break;
	}
	return TRUE;
#endif
}

