/*****************************************************************************
*
* Copyright (C) 2014      Advanced Digital Chips, Inc. All Rights Reserved.
*						http://www.adc.co.kr
*
* THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
* EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE
*
*****************************************************************************/
#include "sdk.h"


#define TIMER_REG_WRITE(bAsync,ch,addr,data)\
	 do{\
			*(vU32*)addr=data;\
			if(bAsync){\
				while(!((*(vU32*)TMUPDATE_ADDR(ch))&1));\
				*(vU32*)TMUPDATE_ADDR(ch) = 1;\
			}\
		}while (0);

#define TIEMR_REG_READ(ch,addr,data)\
	do{\
			data = *(vU32*)addr;\
			if(bAsync){\
				while(!((*(vU32*)TMUPDATE_ADDR(ch))&2);\
				data = *(vU32*)addr;\
				*(vU32*)TMUPDATE_ADDR(ch) = 2;\
			}\
	}while(0);

BOOL set_timer(int nCh, U32 ms)
{
	const int timerintnum[] = { INTNUM_TIMER0,INTNUM_TIMER1,INTNUM_TIMER2,INTNUM_TIMER3,INTNUM_TIMER4 };
	if (nCh > MAX_TIMER_CHANNEL)
		return FALSE;
	U32 r_tpcon = TPCTRL_ADDR(nCh);
	U32 r_tmcon = TMCTRL_ADDR(nCh);
	U32 r_tmcnt = TMCNT_ADDR(nCh);
	int conval;
	int i = 0;
	U32 clk = 0;
	
	U64 nanosecpertick;
	U64 cnt;
	U32 pres;

	U32 scaler[] = { 2,8,32,128,512,2048,8192,32768,0 };

	U32 usec = ms * 1000;
		
	bool bAsync = false;

	if(*(U32*)TMCLK_ADDR(nCh) & 1)
		bAsync = true;
	
	const DEV_CLOCK_TYPE clocktype[MAX_TIMER_CHANNEL + 1] = { DEV_CLOCK_TIMER_0_1,DEV_CLOCK_TIMER_0_1 ,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_4 };
	
	if(bAsync)
		clk = get_clock(clocktype[nCh]);
	else
		clk = get_apb_clock();

	while (1)
	{
		pres = scaler[i];
		if (pres == 0)
			return FALSE;
		nanosecpertick = 1000000000 / (clk / pres);
		cnt = ((U64)usec * 1000) / nanosecpertick;

		if (cnt < 0xffffffffLL) // ms is too big to set, adjust PRESACLE.
			break;
		i++;
	}
	//reset

	TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 1 << 1);
	TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 0);

	conval = i << 1;
	TIMER_REG_WRITE(bAsync, nCh, r_tmcnt, cnt);
	TIMER_REG_WRITE(bAsync, nCh, r_tmcon, conval | 1);
	//*(vU32*)r_tmcon = conval;
	enable_interrupt(timerintnum[nCh], TRUE);
	return TRUE;
}

BOOL stop_timer(int nCh)
{
	if (nCh > MAX_TIMER_CHANNEL)
		return FALSE;

	bool bAsync = false;
	const DEV_CLOCK_TYPE clocktype[MAX_TIMER_CHANNEL + 1] =
	{ DEV_CLOCK_TIMER_0_1,DEV_CLOCK_TIMER_0_1 ,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_4 };

	U32 clk = get_clock(clocktype[nCh]);
	if (clk == OSC_CLOCK)
		bAsync = true;
	U32 r_tmcon = TMCTRL_ADDR(nCh);
	TIMER_REG_WRITE(bAsync, nCh, r_tmcon, 0);//stop
	return TRUE;
}

void set_pwm(int nCh, int hz, int number, bool bperiod, bool en)
{	
	if (nCh > MAX_TIMER_CHANNEL)
		return FALSE;
	
	U32 r_tpcon = TPCTRL_ADDR(nCh);
	U32 r_tmcon = TMCTRL_ADDR(nCh);
	U32 r_tmcnt = TMCNT_ADDR(nCh);
	U32 r_tmdut = TMDUT_ADDR(nCh);
	U32 r_tmpul = TMPUL_ADDR(nCh);
	
	int conval;	
	U32 clk = 0;
	
	bool bAsync = false;

	if(*(U32*)TMCLK_ADDR(nCh) & 1)
		bAsync = true;
	
	const DEV_CLOCK_TYPE clocktype[MAX_TIMER_CHANNEL + 1] = { DEV_CLOCK_TIMER_0_1,DEV_CLOCK_TIMER_0_1 ,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_2_3,DEV_CLOCK_TIMER_4 };
	
	if(bAsync)
		clk = get_clock(clocktype[nCh]);
	else
		clk = get_apb_clock();	
	
	U32 pwm_pre[5] = {512, 128, 32, 8, 2};
	
	if(en)
	{
		TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 1 << 1);
		TIMER_REG_WRITE(bAsync, nCh, r_tpcon, 0);		
		
		int i = 0;
		int prescale = 0;
		int f_prescale = 0;		
		
		for(i=0;i<8;i++)
		{
			if( clk / pwm_pre[i] > hz)
				break;
		}
		
		switch(i)
		{			
			case 0:
				prescale = 512;
				f_prescale = 4 << 1;
				break;
			case 1:
				prescale = 128;
				f_prescale = 3 << 1;
				break;
			case 2:
				prescale = 32;
				f_prescale = 2 << 1;
				break;
			case 3:
				prescale = 8;
				f_prescale = 1 << 1;
				break;
			case 4:
				prescale = 2;
				f_prescale = 0 << 1;
				break;
		}
		
		// TMCNT = ( Set_time(s) * APB Clk(Hz) / (1/Pre-scaler Factor) )	
		int pulse_period =  clk / prescale / hz;		
		TIMER_REG_WRITE(bAsync, nCh, r_tmcnt, pulse_period);		
		TIMER_REG_WRITE(bAsync, nCh, r_tmdut, pulse_period / 2);		
		TIMER_REG_WRITE(bAsync, nCh, r_tmpul, number);				
		
		if(bperiod)
			conval = F_TMCTRL_TMOD_PWM | F_TMCTRL_PWML_HIGH | f_prescale;			
		else
			conval =  F_TMCTRL_TMOD_PWM | F_TMCTRL_PWMO | F_TMCTRL_PWML_HIGH | f_prescale;  // one period
		
		conval |= F_TMCTRL_TMEN;
			
		TIMER_REG_WRITE(bAsync, nCh, r_tmcon, conval);						
	}
	else
	{		
		TIMER_REG_WRITE(bAsync, nCh, r_tmcon, 0);				
	}
}