/*****************************************************************************
*
* Copyright (C) 2014      Advanced Digital Chips, Inc. All Rights Reserved.
*						http://www.adc.co.kr
*
* THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
* EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE
*
*****************************************************************************/
#include "sdk.h"

int twi_set_freq (U8 ch, U32 freq)
{
    unsigned long long tmp;
    U32 twibr0_cnt, twibr1_cnt;

    tmp = (unsigned long long)get_apb_clock()*700/1000000000;    
    twibr0_cnt = tmp + 3;
    twibr1_cnt = (get_apb_clock()/(2*freq)) - ((twibr0_cnt+7) >> 1);
            
	*R_TWIBR0(ch) = twibr0_cnt;
	*R_TWIBR1(ch) = twibr1_cnt;

    return 0;
}

BOOL twi_trans_complete(U8 ch)
{
    U32 status;
	int cnt = TWI_RESP_TIME_OUT_COUNT;
	register volatile U32* pSts asm("%r14")=R_TWISTAT(ch);
	
	while(cnt--)
	{
        status = *pSts;
		if((status & 0x80) == 0x80)
			return TRUE;		
		if(status&0x10){                  // Lost Arbitration
			*R_TWICTRL(ch) = 0;           
			*R_TWISTAT(ch) |= 0x10;    // Clear Lost Arbitration
			debugstring("twi_trans_complete() Lost Arbitration!\r\n");			
			return FALSE;
		}
    }	
	debugstring("twi_trans_complete() time out!\r\n");	
	*R_TWICTRL(ch) = 0;               // Stop twi
	return FALSE;
}

BOOL twi_wait_busy(U8 ch)    /* TWI Status is Busy */
{
	U32 status;
	int cnt = TWI_RESP_TIME_OUT_COUNT;
	
	register volatile U32* pSts asm("%r14")=R_TWISTAT(ch);
	
    while(cnt--)
	{
		status = *pSts ;
		if((status & F_TWISTAT_TWIBUSY)==0)
			return TRUE;
		
		if(status&0x10){                  // Lost Arbitration
			*R_TWICTRL(ch) = 0;           
			*R_TWISTAT(ch) |= 0x10;    // Clear Lost Arbitration
			debugstring("twi_trans_complete() Lost Arbitration!\r\n");
			break;
		}
	}
	debugstring("twi_wait_busy() time out!\r\n");			
	*R_TWISTAT(ch) &= ~(1<<5);  // Clear busy status	
	*R_TWICTRL(ch) = 0;         // Stop twi
	return FALSE;

}

/*
 * devaddr : Base of DEVICE ADDRESS
 * waddr : Buffer of Word Address
 * len : length of 'waddr', 1 or 2
 * flags : TWI_TRANSMIT, TWI_RECEIVE
 */
BOOL twi_start (U8 ch, U8 devaddr, U8 *waddr, int len, int flags)
{
    int i;
    U8 con = 0;
    U16 status;
    con = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWITR_MTRM;
		
	*R_TWICTRL(ch) = con;	 
	*R_TWIDATA(ch) = devaddr;	
	*R_TWIMADR(ch) = (1<<8) | devaddr;			 
	
	if(FALSE == twi_trans_complete(ch))
	{		
		return FALSE;
	}
	
    for (i = 0; i < len; i++) {
		*R_TWIDATA(ch) = waddr[i];		
        if(FALSE == twi_trans_complete(ch))
		{			
			return FALSE;
		}
    } 
	
    if (flags == TWI_RECEIVE) {
        con = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_REPST_RSTART;
		
		*R_TWICTRL(ch) = con;				
		*R_TWIDATA(ch) = devaddr + 1;		
		if(FALSE == twi_trans_complete(ch))
		{			
			return FALSE;
		}
        status  = *R_TWIDATA(ch);     // Read for SCL Generation
			
        if(FALSE == twi_trans_complete(ch))
		{
			return FALSE;
		}
    }
	
	return TRUE;
}

/*
 * devaddr : Base of DEVICE ADDRESS
 * waddr : Buffer of Word Address
 * len : length of 'waddr', 1 or 2
 * buffer : Writing buffer
 * length : Size of Writing
 */
int twi_write(U8 ch, U8 devaddr, U8 *waddr,
        int len, U8 *buffer, int length)
{
    int write_num, i;

    write_num = 0;	
    twi_start (ch, devaddr, waddr, len, TWI_TRANSMIT);
	for (i = 0; i < length; i++) {		
		*R_TWIDATA(ch) = buffer[i];// When word is 8bit, use this		
        if(FALSE == twi_trans_complete(ch))
		{
			return -1;
		}		
        write_num++;
    }
	*R_TWIMADR(ch) = devaddr; // Stop condition	
	delayms(1);
	*R_TWICTRL(ch) = 0;
    return write_num;
}

/*
* devaddr : Base of DEVICE ADDRESS
* waddr : Byte Address
* buffer : Writing buffer
* length : Size of Writing
*/
int twi_write2(U8 ch, U8 devaddr, U8 waddr, U8 *buffer, int length)
{
	int write_num, i;

	write_num = 0;

	twi_start (ch, devaddr, &waddr, 1, TWI_TRANSMIT);
	for (i = 0; i < length; i++) {
		*R_TWIDATA(ch) = buffer[i];
		
		if(FALSE == twi_trans_complete(ch))
		{			
			return -1;
		}
		
		write_num++;
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN;
	*R_TWIMADR(ch) = (devaddr>> 1); // Stop condition
		
	if(FALSE == twi_wait_busy(ch))
	{
		return -1;
	}

	return write_num;
}

/*
 * devaddr : Base of DEVICE ADDRESS
 * addr : Word Address
 * len : length of Word Address (1 or 2)
 * buffer : read buffer
 * length : length of buffer
 */
int twi_read(U8 ch, U8 devaddr, U8 *waddr,
        int len, U8 *buffer, int length)
{
    U32 i;
	U8	tmp;
    int read_cnt = 0;

    twi_start (ch, devaddr, waddr, len, TWI_RECEIVE);

	for ( i = 0; i < length-1; i++ ) {
		buffer[i] = *R_TWIDATA(ch);
		
        if(FALSE == twi_trans_complete(ch))
		{
			return -1;
		}
		
        read_cnt++;
    }
	
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWIAK_NACK;
    buffer[i] = *R_TWIDATA(ch);
	
    if(FALSE == twi_trans_complete(ch))
	{
		return -1;
	}
    read_cnt++;
    tmp = *R_TWIDATA(ch);    // dummy read 

	*R_TWIMADR(ch) = devaddr; // Stop condition
	delayms(1);	
	*R_TWICTRL(ch) = 0;
		
	tmp = *R_TWIDATA(ch);    // dummy read 
	
    if(FALSE == twi_wait_busy(ch))
	{
		return -1;
	}
    
	return read_cnt;
}
