/*****************************************************************************
*
* Copyright (C) 2014      Advanced Digital Chips, Inc. All Rights Reserved.
*						http://www.adc.co.kr
*
* THIS SOFTWARE IS PROVIDED BY ADCHIPS "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
* EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ADCHIPS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE
*
*****************************************************************************/
#include "sdk.h"

int twi_set_freq (U8 ch, U32 freq)
{
    unsigned long long tmp;
    U32 twibr0_cnt, twibr1_cnt;

    tmp = (unsigned long long)get_apb_clock()*700/1000000000;    
    twibr0_cnt = tmp + 3;
    twibr1_cnt = (get_apb_clock()/(2*freq)) - ((twibr0_cnt+7) >> 1);
            
	*R_TWIBR0(ch) = twibr0_cnt;
	*R_TWIBR1(ch) = twibr1_cnt;

    return 0;
}

BOOL TwiCompleteCheck(U8 ch, U32 wait_time)
{// 0 -> OK, 1->Error
    U8 state=1,aa;
		U32 tick = get_tick_count();
	while ((get_tick_count() - tick) < wait_time) {
		aa = *R_TWISTAT(ch);
		if (aa & 0x80) {
			state = 0;
			return state;
		}
		wait_time--;

		if (aa & 0x10) {                       // Lost Arbitration
			*R_TWICTRL(ch) = 0;
			*R_TWISTAT(ch) |= 0x10;    // Clear Lost Arbitration
			debugprintf("Lost Arbitration\r\n");
			break;
		}
	}
	debugstring("twi_trans_complete() time out!\r\n");	
	//*R_TWICTRL(ch) = 0;               // Stop twi
	return state;
}

BOOL TwiBusBusyCheck(U8 ch, U32 wait_time)    /* TWI Status is Busy */
{//0 -> OK, 1 -> Error
	U8 state = 1, aa;
	U32 tick = get_tick_count();
	while ((get_tick_count() - tick) < wait_time) {
		aa = *R_TWISTAT(ch);
		if ((aa & 0x20) == 0) {
			state = 0;
			return state;
		}
		wait_time--;

		if (aa & 0x10) {                       // Lost Arbitration
			*R_TWICTRL(ch) = 0;
			*R_TWISTAT(ch) |= 0x10;    // Clear Lost Arbitration			
			debugprintf("Lost Arbitration\r\n");
			break;
		}
	}
	*R_TWISTAT(ch) &= ~(1 << 5);  // Clear busy status
	debugprintf("TWI Timeout in Busy Check\r\n");
	return state;
}

static U8 TwiStop(int ch,U8 address)
{
	*R_TWIMADR(ch) = address;
	*R_TWICTRL(ch) = 0;
	*R_TWIDATA(ch) = 0xfe;
	delayms(10);
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWITR_MTRM | F_TWICTRL_TWIAK_NACK;
	delayms(1);
	return 0;
}

/*
* ch : TWI channel
* devaddr : Base of DEVICE ADDRESS
* reg : register or memory Address
* len : length of reg
* data : Writing data buffer
* length : length of data
*/
 
bool twi_write_multi(U8 ch, U8 devaddr, U8 *reg,  int len, U8 *data, int length)
{	
		if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		if (TwiBusBusyCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWITR_MTRM;
	*R_TWIDATA(ch) = devaddr;
	*R_TWIMADR(ch) = (1 << 8) | devaddr; // // start condition, Device ID Write	

	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}

	int i;
	for (i = 0; i < len; i++)
	{
		*R_TWIDATA(ch) = reg[i];// register (address) write
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}	

	for (i = 0; i < len; i++)
	{
		*R_TWIDATA(ch) = reg[i];// data write
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}	
	
	// stop condition
	*R_TWIMADR(ch) = 0;
	if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN;	

	return true;
}

/*
* ch : TWI channel
* devaddr : Base of DEVICE ADDRESS
* reg : register or memory Address
* data : Writing data 
*/

int twi_write(U8 ch, U8 devaddr, U8 reg, U8 data)
{
		if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		if (TwiBusBusyCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWITR_MTRM;
	*R_TWIDATA(ch) = devaddr;
	*R_TWIMADR(ch) = (1 << 8) | devaddr;  // start condition, Device ID Write	

	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}

	*R_TWIDATA(ch) = reg;// data write
	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}

	*R_TWIDATA(ch) = data;// data write
	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}

	// stop condition
	*R_TWIMADR(ch) = 0;
	if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN;
	
	return true;
}

/*
 * devaddr : Base of DEVICE ADDRESS
 * addr : Word Address
 * len : length of Word Address (1 or 2)
 * buf : read buffer
 * length : length of buffer
 */
 
int twi_read(U8 ch, U8 devaddr, U8 *reg, int len, U8 *buf, int length)
{
	if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		if (TwiBusBusyCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}
	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_TWITR_MTRM;
	*R_TWIDATA(ch) = devaddr;
	*R_TWIMADR(ch) = (1 << 8) | devaddr; // // start condition, Device ID Write	

	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}

	int i = 0;
	
	for (i = 0; i < len; i++)
	{
		*R_TWIDATA(ch) = reg[i];// register (devaddr) write
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}	

	*R_TWICTRL(ch) = F_TWICTRL_TWIEN | F_TWICTRL_TWIMOD_MSTART | F_TWICTRL_REPST_RSTART;
	*R_TWIDATA(ch) = devaddr | 0x01;

	if (TwiCompleteCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}
	
	if (length == 1)
	{
		//last data
		*R_TWICTRL(ch) |= F_TWICTRL_TWIAK_NACK;
		buf[0] = *R_TWIDATA(ch);//dummy read
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}

	}
	else
	{
		buf[i] = *R_TWIDATA(ch);//dummy read for clock gen
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
		for (i = 0; i < (length - 2); i++)
		{
			buf[i] = *R_TWIDATA(ch);
			if (TwiCompleteCheck(ch,100)) {
				TwiStop(ch,devaddr);
				return false;
			}
		}
		//last data
		*R_TWICTRL(ch) |= F_TWICTRL_TWIAK_NACK;
		buf[i++] = *R_TWIDATA(ch);
		if (TwiCompleteCheck(ch,100)) {
			TwiStop(ch,devaddr);
			return false;
		}
	}

	// stop condition
	*R_TWIMADR(ch) = 0;
	if (TwiBusBusyCheck(ch,100)) {
		TwiStop(ch,devaddr);
		return false;
	}
	*R_TWICTRL(ch) = 0; // don't gen clock
	buf[i] = *R_TWIDATA(ch); //real data

	*R_TWICTRL(ch) = F_TWICTRL_TWIEN;
	return i;
}
