
#include "sdk.h"

///////////////////////////
// USB Bulk
///////////////////////////

const U8 USB_Device_Descriptor[] =
{
    DEV_LEN,	        // bLength: Size of descriptor
    0x01,				// bDescriptorType: Device	
    0x10,0x01,			// bcdUSB: USB 1.1
    0x00,				// bDeviceClass: none
    0x00,				// bDeviceSubClass: none
    0x00,				// bDeviceProtocol: none
    EP0_LEN,			// bMaxPacketSize0: 16 bytes
    0xDC,0x0A,			// idVendor: 0x0ADC 
    0x30,0x00,  // idProduct: 
    0x01,0x00,			// bcdDevice: device release
    0x00,				// iManufacturer: 
    0x00,				// iProduct: 
    0x00,				// iSerialNumber: none
    0x01				// bnumconfigurations: 1
};

const U8 USB_Configuration_Descriptor[] =
{
 //=====================================================================
 // Configuration

    0x09,		        // bLength: Size of descriptor
    0x02,				// bDescriptorType: Configuration	
    0x24,0x00,		// wTotalLength: Cfg+Ifc+Class+Ep = 46 bytes
    0x01,				// bNumInterfaces: 1 interface
    0x01,				// bConfigurationValue: 1
    0x00,				// iConfiguration: none
    0x40,				// bmAttributes: Self powered 
    0xFA,				// MaxPower: 500mA
//----------------------------------------------------------------------
// INTERFACE DESCRIPTOR
//----------------------------------------------------------------------
    0x09,				// bLength: Size of descriptor
    0x04,				// bDescriptorType: Interface
    0x00,				// bInterfaceNumber: #1
    0x00,				// bAlternateSetting: #0
    0x02,				// bNumEndpoints: 4
    0x00,				// bInterfaceClass: HID-class
    0x00,				// bInterfaceSubClass: none
    0x00,				// bInterfaceProtocol: none
    0x00,				// iInterface: none
//----------------------------------------------------------------------
// ENDPOINT 1 DESCRIPTOR  (BULK OUT)
//----------------------------------------------------------------------
    0x09,				// bLength: Size of descriptor
    0x05,				// bDescriptorType: Endpoint
    0x01,				// bEndpointAddress: OUT, EP1
    0x02,				// bmAttributes: Bulk
    EP1_LEN,0x00,		// wMaxPacketSize: 
    0x01,	            // bInterval: 1ms  , ignored for Bulk
    0x00,0x00,			//   Ǵ dummy ̴. WriteEP0 ó ϱ  ̸ ξ

//----------------------------------------------------------------------
// ENDPOINT 2 DESCRIPTOR  (BULK IN)
//----------------------------------------------------------------------
    0x09,				// bLength: Size of descriptor
    0x05,				// bDescriptorType: Endpoint
    0x82,				// bEndpointAddress: IN, EP2
    0x02,				// bmAttributes: Bulk
    EP2_LEN,0x00,		// wMaxPacketSize: 
    0x01,	            // bInterval: 1ms, ignored for Bulk 
    0x00,0x00			//   Ǵ dummy ̴. WriteEP0 ó ϱ  ̸ ξ
};

///////////////////////////
// USB Virtual Com
//////////////////////////

const U8 USB_Virtual_DeviceDescriptor[VIRTUAL_COM_PORT_SIZ_DEVICE_DESC] =
{
    0x12,   /* bLength */
    0x01,     /* bDescriptorType */
    0x10, 0x01,   /* bcdUSB = 1.1 */
    0x02,   /* bDeviceClass: CDC */
    0x00,   /* bDeviceSubClass */
    0x00,   /* bDeviceProtocol */
    0x10,   /* bMaxPacketSize0 */
    0xDC,
    0x0A,   /* idVendor = 0x0ADC */
    0x20,
    0x01,   /* idProduct = 0x0120 */
    0x00,
    0x02,   /* bcdDevice = 2.00 */
    1,              /* Index of string descriptor describing manufacturer */
    2,              /* Index of string descriptor describing product */
    3,              /* Index of string descriptor describing the device's serial number */
    0x01    /* bNumConfigurations */
};

const U8 USB_Virtual_ConfigDescriptor[VIRTUAL_COM_PORT_SIZ_CONFIG_DESC] =
{
    /*Configuation Descriptor*/
    0x09,   /* bLength: Configuation Descriptor size */
    0x02,      /* bDescriptorType: Configuration */
    VIRTUAL_COM_PORT_SIZ_CONFIG_DESC,       /* wTotalLength:no of returned bytes */
    0x00,
    0x02,   /* bNumInterfaces: 2 interface */
    0x01,   /* bConfigurationValue: Configuration value */
    0x00,   /* iConfiguration: Index of string descriptor describing the configuration */
    0xC0,   /* bmAttributes: self powered */
    0xFA,   /* MaxPower 500 mA */

    /*Interface Descriptor*/
    0x09,   /* bLength: Interface Descriptor size */
    0x04,  /* bDescriptorType: Interface */

    /* Interface descriptor type */
    0x00,   /* bInterfaceNumber: Number of Interface */
    0x00,   /* bAlternateSetting: Alternate setting */
    0x01,   /* bNumEndpoints: One endpoints used */
    0x02,   /* bInterfaceClass: Communication Interface Class */
    0x02,   /* bInterfaceSubClass: Abstract Control Model */
    0x01,   /* bInterfaceProtocol: Common AT commands */
    0x00,   /* iInterface: */

    /*Header Functional Descriptor*/
    0x05,   /* bLength: Endpoint Descriptor size */
    0x24,   /* bDescriptorType: CS_INTERFACE */
    0x00,   /* bDescriptorSubtype: Header Func Desc */
    0x10,   /* bcdCDC: spec release number */
    0x01,

    /*Call Managment Functional Descriptor*/
    0x05,   /* bFunctionLength */
    0x24,   /* bDescriptorType: CS_INTERFACE */
    0x01,   /* bDescriptorSubtype: Call Management Func Desc */
    0x00,   /* bmCapabilities: D0+D1 */
    0x01,   /* bDataInterface: 1 */
	
    /*ACM Functional Descriptor*/
    0x04,   /* bFunctionLength */
    0x24,   /* bDescriptorType: CS_INTERFACE */
    0x02,   /* bDescriptorSubtype: Abstract Control Management desc */
    0x02,   /* bmCapabilities */
	
    /*Union Functional Descriptor*/
    0x05,   /* bFunctionLength */
    0x24,   /* bDescriptorType: CS_INTERFACE */
    0x06,   /* bDescriptorSubtype: Union func desc */
    0x00,   /* bMasterInterface: Communication class interface */
    0x01,   /* bSlaveInterface0: Data Class Interface */

    /*Endpoint 4 Descriptor*/
    0x07,   /* bLength: Endpoint Descriptor size */
    0x05,   /* bDescriptorType: Endpoint */
    0x84,   /* bEndpointAddress: (IN4) */
    0x03,   /* bmAttributes: Interrupt */
    VIRTUAL_COM_PORT_INT_SIZE,      /* wMaxPacketSize: */
    0x00,
    0xFF,   /* bInterval: */

    /*Data class interface descriptor*/
    0x09,   /* bLength: Endpoint Descriptor size */
    0x04,  /* bDescriptorType: */
    0x01,   /* bInterfaceNumber: Number of Interface */
    0x00,   /* bAlternateSetting: Alternate setting */
    0x02,   /* bNumEndpoints: Two endpoints used */
    0x0A,   /* bInterfaceClass: CDC */
    0x00,   /* bInterfaceSubClass: */
    0x00,   /* bInterfaceProtocol: */
    0x00,   /* iInterface: */

    /*Endpoint 1 Descriptor*/
    0x07,   /* bLength: Endpoint Descriptor size */
    0x05,   /* bDescriptorType: Endpoint */
    0x01,   /* bEndpointAddress: (OUT1) */
    0x02,   /* bmAttributes: Bulk */
    VIRTUAL_COM_PORT_DATA_SIZE,      /* wMaxPacketSize: */
    0x00,
    0x00,   /* bInterval: ignore for Bulk transfer */

    /*Endpoint 2 Descriptor*/
    0x07,   /* bLength: Endpoint Descriptor size */
    0x05,   /* bDescriptorType: Endpoint */
    0x82,   /* bEndpointAddress: (IN2) */
    0x02,   /* bmAttributes: Bulk */
    VIRTUAL_COM_PORT_DATA_SIZE,             /* wMaxPacketSize: */
    0x00,
    0x00,   /* bInterval: ignore for Bulk transfer */
};

/* USB String Descriptors */
const U8 Virtual_Com_Port_StringLangID[VIRTUAL_COM_PORT_SIZ_STRING_LANGID] =
{
    VIRTUAL_COM_PORT_SIZ_STRING_LANGID,
    0x03,
    0x09,
    0x04 /* LangID = 0x0409: U.S. English */
};

const U8 Virtual_Com_Port_StringVendor[VIRTUAL_COM_PORT_SIZ_STRING_VENDOR] =
{
    VIRTUAL_COM_PORT_SIZ_STRING_VENDOR,     /* Size of Vendor string */
    0x03,             /* bDescriptorType*/
    'A', 0, 'd', 0, 'v', 0, 'a', 0, 'n', 0, 'c', 0, 'e', 0, 'd', 0,
    ' ', 0, 'D', 0, 'i', 0, 'g', 0, 'i', 0, 't', 0, 'a', 0, 'l', 0,
    ' ', 0, 'c', 0, 'h', 0, 'i', 0, 'p', 0, 's', 0, 'e', 0,
};

const U8 Virtual_Com_Port_StringProduct[VIRTUAL_COM_PORT_SIZ_STRING_PRODUCT] =
{
    VIRTUAL_COM_PORT_SIZ_STRING_PRODUCT,          /* bLength */
    0x03,        /* bDescriptorType */
    'a', 0, 'd', 0, 'c', 0, 'h', 0, 'i', 0, 'p', 0, 's', 0, ' ', 0,
    'V', 0, 'i', 0, 'r', 0, 't', 0, 'u', 0, 'a', 0, 'l', 0, ' ', 0, 
    'C', 0, 'O', 0, 'M', 0, ' ', 0, 'P', 0, 'o', 0, 'r', 0, 't', 0, 
};

U8 Virtual_Com_Port_StringSerial[VIRTUAL_COM_PORT_SIZ_STRING_SERIAL] =
{
    VIRTUAL_COM_PORT_SIZ_STRING_SERIAL,           /* bLength */
    0x03,                   /* bDescriptorType */
    'a', 0, 'd', 0, 'c', 0, 'h', 0, 'i', 0, 'p', 0, 's', 0
};

///////////////
// USB HID
//////////////

//===================================================================================================
//	DEVICE DESCRIPTOR
//===================================================================================================
const U8 USB_HID_Device_Descriptor[] =
{
    DESCRIPTOR_SIZE_DEVICE,				// bLength: Size of descriptor
    DESCRIPTOR_TYPE_DEVICE,				// bDescriptorType: Device
    WORD_TO_BYTE(0x100),				// bcdUSB: USB 1.1
    0,									// bDeviceClass: none
    0,									// bDeviceSubClass: none
    0,									// bDeviceProtocol: none
    EP0_LEN,							// bMaxPacketSize0: 16 bytes
    WORD_TO_BYTE(0xADC),				// idVendor: 0xADC
//---------------------------------------------------------------------------------------------------
    WORD_TO_BYTE(0x300),				// idProduct: 0x300
//---------------------------------------------------------------------------------------------------
    WORD_TO_BYTE(0x1),				// bcdDevice: device release
//---------------------------------------------------------------------------------------------------
    0x01,								// iManufacturer: 1'st String Descriptor
    0x02,								// iProduct: 2'nd String Descriptor
    0x03,								// iSerialNumber: 3'rd String Descriptor
//---------------------------------------------------------------------------------------------------
    0x01								// bnumconfigurations: 1
};
const U8 USB_Device_Des_size = (sizeof(USB_Device_Descriptor));

volatile U32 usb_hid_in_len=HID_INPUT_REPORT_BYTES;
volatile U32 usb_hid_out_len=HID_OUTPUT_REPORT_BYTES;

const U8 HID_ReportDescriptor[] =
{
	HID_UsagePageVendor	(0x00),
    HID_Usage          	(0x01),
    HID_Collection     	(HID_Application),
//---------------------------------------------------------------------------------------------------
	//HID_ReportID		(USB_HID_IN_REPORT_ID),
    HID_Usage			(0x03),
	HID_LogicalMin		(0),  										// value range: 0 - 0xFF
    HID_LogicalMaxS		(0xFF),
    HID_ReportSize		(8),  										// 8 bits
    HID_ReportCount		(HID_INPUT_REPORT_BYTES),
    HID_Input			(HID_Data | HID_Variable | HID_Absolute),
//---------------------------------------------------------------------------------------------------
    //HID_ReportID		(USB_HID_OUT_REPORT_ID),
	HID_Usage			(0x04),
	HID_LogicalMin		(0),  										// value range: 0 - 0xFF
    HID_LogicalMaxS		(0xFF),
    HID_ReportSize		(8),  										// 8 bits
    HID_ReportCount		(HID_OUTPUT_REPORT_BYTES),
    HID_Output			(HID_Data | HID_Variable | HID_Absolute),
//---------------------------------------------------------------------------------------------------
	HID_Usage			(0x05),
	HID_LogicalMin		(0),  										// value range: 0 - 0xFF
    HID_LogicalMaxS		(0xFF),
    HID_ReportSize		(8),  										// 8 bits
    HID_ReportCount		(HID_FEATURE_REPORT_BYTES ),
	HID_Feature			(HID_Data | HID_Variable | HID_Absolute),
//---------------------------------------------------------------------------------------------------
    HID_EndCollection
};
#define HID_REPORT_DES_SIZE (sizeof(HID_ReportDescriptor))
const U16 HID_Report_Des_size = (sizeof(HID_ReportDescriptor));

const U8 USB_HID_Configuration_Descriptor[] =
{
//===================================================================================================
// CONFIGURATION DESCRIPTOR
//===================================================================================================

    DESCRIPTOR_SIZE_CONFIGURATION,	// bLength: Size of descriptor
    DESCRIPTOR_TYPE_CONFIGURATION,	// bDescriptorType: Configuration
    WORD_TO_BYTE (
        DESCRIPTOR_SIZE_CONFIGURATION +
        DESCRIPTOR_SIZE_INTERFACE +
        DESCRIPTOR_SIZE_HID +
		DESCRIPTOR_SIZE_ENDPOINT
    ),								// wTotalLength
//---------------------------------------------------------------------------------------------------
    0x01,							// bNumInterfaces: 1 interface
//---------------------------------------------------------------------------------------------------
    0x01,							// bConfigurationValue: 1
    0x00,							// iConfiguration: none
    USB_BUS_POWER,					// bmAttributes: Self powered
    USB_MAX_POWER,					// MaxPower:
//---------------------------------------------------------------------------------------------------

//===================================================================================================
// INTERFACE DESCRIPTOR
//===================================================================================================
    DESCRIPTOR_SIZE_INTERFACE,					// bLength: Size of descriptor
    DESCRIPTOR_TYPE_INTERFACE,			// bDescriptorType: Interface
    0x00,							// bInterfaceNumber: #1
    0x00,							// bAlternateSetting: #0
//---------------------------------------------------------------------------------------------------
    0x01,							// bNumEndpoints: 1	Interrupt EP4
    DEVICE_CLASS_HUMAN_INTERFACE,	// bInterfaceClass: HID-class
//---------------------------------------------------------------------------------------------------
    0x00,							// bInterfaceSubClass: none
    0x00,							// bInterfaceProtocol: none
//---------------------------------------------------------------------------------------------------
    0x04,							// iInterface:
//---------------------------------------------------------------------------------------------------

//===================================================================================================
// HID CLASS DESCRIPTOR
//===================================================================================================
    DESCRIPTOR_SIZE_HID,			/* bLength */
    HID_HID_DESCRIPTOR_TYPE,		/* bDescriptorType */
    WORD_TO_BYTE(0x100),    		/* bcdHID 1.00 */
    0x00,                  			/* bCountryCode */
    0x01,                   		/* bNumDescriptors */
    HID_REPORT_DESCRIPTOR_TYPE,		/* bDescriptorType */
    WORD_TO_BYTE(HID_REPORT_DES_SIZE),
//===================================================================================================
// Endpoint 4, INTERRUPT IN
//===================================================================================================
    DESCRIPTOR_SIZE_ENDPOINT,		/* bLength */
    DESCRIPTOR_TYPE_ENDPOINT,   	/* bDescriptorType */
    0x84,                			/* bEndpointAddress */
    0x03,       					/* bmAttributes */
    WORD_TO_BYTE(EP4_LEN),		/* wMaxPacketSize */
    // 0x20,/* 32ms */					/* bInterval */
    0x0A,/* 10ms */					/* bInterval */
//---------------------------------------------------------------------------------------------------
};
const U8 USB_Configuration_Des_size = (sizeof(USB_Configuration_Descriptor));

const U8 USB_StringDescriptor[] =
{
    // Index 0x00: LANGID Codes
	DESCRIPTOR_SIZE_STRING,		/* bLength */
    DESCRIPTOR_TYPE_STRING,		/* bDescriptorType */
    0x09,0x04,					/* US English *//* wLANGID */

    // Index 0x01: Manufacturer
    (13*2 + 2),					/* bLength (13 Char + Type + lenght) */
    DESCRIPTOR_TYPE_STRING,		/* bDescriptorType */
    'A',0,
    'D',0,
    'C',0,
    ' ',0,
    ' ',0,
    'S',0,
    'o',0,
    'f',0,
    't',0,
    'w',0,
    'a',0,
    'r',0,
    'e',0,

    // Index 0x02: Product
    (18*2 + 2),   				/* bLength (18 Char + Type + lenght) */
    DESCRIPTOR_TYPE_STRING,		/* bDescriptorType */
    'a',0,
    'd',0,
    'L',0,
    'u',0,
    'n',0,
    'a',0,
    ' ',0,
    'S',0,
    'T',0,
    'K',0,
    ' ',0,
    'U',0,
    'S',0,
    'B',0,
    ' ',0,
    'H',0,
    'I',0,
    'D',0,

    // Index 0x03: Serial Number
    (12*2 + 2),   				/* bLength (12 Char + Type + lenght) */
    DESCRIPTOR_TYPE_STRING,		/* bDescriptorType */
    'A',0,        				/* allowed characters are       */
    'B',0,        				/*   0x0030 - 0x0039 ('0'..'9') */
    'C',0,        				/*   0x0041 - 0x0046 ('A'..'F') */
    '1',0,        				/*   length >= 26               */
    '2',0,
    '3',0,
    '4',0,
    '5',0,
    '6',0,
    '7',0,
    '8',0,
    '9',0,

    // Index 0x04: Interface 0, Alternate Setting 0
    ( 3*2 + 2),   				/* bLength (3 Char + Type + lenght) */
    DESCRIPTOR_TYPE_STRING,		/* bDescriptorType */
    'H',0,
    'I',0,
    'D',0,
};
const U8 USB_String_Des_size = sizeof(USB_StringDescriptor);
// #endif

